@@ -28,12 +28,12 @@
 
 #include <config.h>
 #include "ovs-lldp.h"
+#include <sys/types.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <inttypes.h>
-#include <netinet/in.h>
 #include <stdbool.h>
 #include <stdlib.h>
-#include <sys/types.h>
 #include "openvswitch/dynamic-string.h"
 #include "flow.h"
 #include "openvswitch/list.h"
@@ -42,7 +42,7 @@
 #include "netdev.h"
 #include "openvswitch/types.h"
 #include "packets.h"
-#include "poll-loop.h"
+#include "openvswitch/poll-loop.h"
 #include "smap.h"
 #include "unixctl.h"
 #include "util.h"
@@ -101,21 +101,6 @@ static struct hmap *const all_mappings OVS_GUARDED_BY(mutex) = &all_mappings__;
 
 static struct lldp_aa_element_system_id system_id_null;
 
-/* Convert an LLDP chassis ID to a string.
- */
-static void
-chassisid_to_string(uint8_t *array, size_t len, char **str)
-{
-    unsigned int i;
-
-    *str = xmalloc(len * 3);
-
-    for (i = 0; i < len; i++) {
-        snprintf(&(*str)[i * 3], 4, "%02x:", array[i]);
-    }
-    (*str)[(i * 3) - 1] = '\0';
-}
-
 /* Find an Auto Attach mapping keyed by I-SID.
  */
 static struct aa_mapping_internal *
@@ -180,16 +165,16 @@ aa_print_lldp_and_aa_stats(struct ds *ds, struct lldp *lldp)
     }
 
     LIST_FOR_EACH (hw, h_entries, &lldp->lldpd->g_hardware) {
-        ds_put_format(ds, "\ttx cnt: %"PRIu64"\n", hw->h_tx_cnt);
-        ds_put_format(ds, "\trx cnt: %"PRIu64"\n", hw->h_rx_cnt);
-        ds_put_format(ds, "\trx discarded cnt: %"PRIu64"\n",
+        ds_put_format(ds, "  tx cnt: %"PRIu64"\n", hw->h_tx_cnt);
+        ds_put_format(ds, "  rx cnt: %"PRIu64"\n", hw->h_rx_cnt);
+        ds_put_format(ds, "  rx discarded cnt: %"PRIu64"\n",
                       hw->h_rx_discarded_cnt);
-        ds_put_format(ds, "\trx unrecognized cnt: %"PRIu64"\n",
+        ds_put_format(ds, "  rx unrecognized cnt: %"PRIu64"\n",
                       hw->h_rx_unrecognized_cnt);
-        ds_put_format(ds, "\tageout cnt: %"PRIu64"\n", hw->h_ageout_cnt);
-        ds_put_format(ds, "\tinsert cnt: %"PRIu64"\n", hw->h_insert_cnt);
-        ds_put_format(ds, "\tdelete cnt: %"PRIu64"\n", hw->h_delete_cnt);
-        ds_put_format(ds, "\tdrop cnt: %"PRIu64"\n", hw->h_drop_cnt);
+        ds_put_format(ds, "  ageout cnt: %"PRIu64"\n", hw->h_ageout_cnt);
+        ds_put_format(ds, "  insert cnt: %"PRIu64"\n", hw->h_insert_cnt);
+        ds_put_format(ds, "  delete cnt: %"PRIu64"\n", hw->h_delete_cnt);
+        ds_put_format(ds, "  drop cnt: %"PRIu64"\n", hw->h_drop_cnt);
     }
 }
 
@@ -204,30 +189,31 @@ aa_print_element_status_port(struct ds *ds, struct lldpd_hardware *hw)
                    sizeof port->p_element.system_id)) {
             const char *none_str = "<None>";
             const char *descr = NULL;
-            char *id = NULL;
-            char *system;
+            struct ds id = DS_EMPTY_INITIALIZER;
+            struct ds system = DS_EMPTY_INITIALIZER;
 
             if (port->p_chassis) {
                 if (port->p_chassis->c_id_len > 0) {
-                    chassisid_to_string(port->p_chassis->c_id,
-                                        port->p_chassis->c_id_len, &id);
+                    ds_put_hex_with_delimiter(&id, port->p_chassis->c_id,
+                                   port->p_chassis->c_id_len, ":");
                 }
 
                 descr = port->p_chassis->c_descr;
             }
 
-            chassisid_to_string((uint8_t *) &port->p_element.system_id,
-                sizeof port->p_element.system_id, &system);
+            ds_put_hex_with_delimiter(&system,
+                                      (uint8_t *) &port->p_element.system_id,
+                                      sizeof port->p_element.system_id, ":");
 
-            ds_put_format(ds, "\tAuto Attach Primary Server Id: %s\n",
-                          id ? id : none_str);
-            ds_put_format(ds, "\tAuto Attach Primary Server Descr: %s\n",
+            ds_put_format(ds, "  Auto Attach Primary Server Id: %s\n",
+                          id.length ? ds_cstr_ro(&id) : none_str);
+            ds_put_format(ds, "  Auto Attach Primary Server Descr: %s\n",
                           descr ? descr : none_str);
-            ds_put_format(ds, "\tAuto Attach Primary Server System Id: %s\n",
-                          system);
+            ds_put_format(ds, "  Auto Attach Primary Server System Id: %s\n",
+                          id.length ? ds_cstr_ro(&system) : none_str);
 
-            free(id);
-            free(system);
+            ds_destroy(&id);
+            ds_destroy(&system);
         }
     }
 }
@@ -391,7 +377,7 @@ update_mapping_on_lldp(struct lldp *lldp, struct lldpd_hardware *hardware,
 {
     struct lldpd_aa_isid_vlan_maps_tlv *lm = xzalloc(sizeof *lm);
 
-    VLOG_INFO("\t\t hardware->h_ifname=%s", hardware->h_ifname);
+    VLOG_INFO("     hardware->h_ifname=%s", hardware->h_ifname);
 
     lm->isid_vlan_data.isid = m->isid;
     lm->isid_vlan_data.vlan = m->vlan;
@@ -526,7 +512,7 @@ aa_mapping_register(void *aux, const struct aa_mapping_settings *s)
         struct lldpd_hardware *hw;
         struct aa_mapping_internal *m;
 
-        VLOG_INFO("\t lldp->name=%s", lldp->name);
+        VLOG_INFO("   lldp->name=%s", lldp->name);
 
         if (mapping_find_by_isid(lldp, s->isid)) {
             continue;
@@ -559,14 +545,14 @@ aa_mapping_unregister_mapping(struct lldp *lldp,
                               struct lldpd_hardware *hw,
                               struct aa_mapping_internal *m)
 {
-    struct lldpd_aa_isid_vlan_maps_tlv *lm, *lm_next;
+    struct lldpd_aa_isid_vlan_maps_tlv *lm;
 
-    LIST_FOR_EACH_SAFE (lm, lm_next, m_entries,
+    LIST_FOR_EACH_SAFE (lm, m_entries,
                         &hw->h_lport.p_isid_vlan_maps) {
         uint32_t isid = lm->isid_vlan_data.isid;
 
         if (isid == m->isid) {
-            VLOG_INFO("\t\t Removing lport, isid=%u, vlan=%u",
+            VLOG_INFO("     Removing lport, isid=%u, vlan=%u",
                       isid,
                       lm->isid_vlan_data.vlan);
 
@@ -609,7 +595,7 @@ aa_mapping_unregister(void *aux)
             uint16_t vlan = m->vlan;
             struct aa_mapping_internal *p = mapping_find_by_isid(lldp, isid);
 
-            VLOG_INFO("\t Removing mapping ISID=%"PRIu32", VLAN=%"PRIu16
+            VLOG_INFO("   Removing mapping ISID=%"PRIu32", VLAN=%"PRIu16
                       " (lldp->name=%s)", isid, vlan, lldp->name);
 
             if (p) {
@@ -620,7 +606,7 @@ aa_mapping_unregister(void *aux)
 
             /* Remove from all the lldp instances */
             LIST_FOR_EACH (hw, h_entries, &lldp->lldpd->g_hardware) {
-                VLOG_INFO("\t\t hardware->h_ifname=%s", hw->h_ifname);
+                VLOG_INFO("     hardware->h_ifname=%s", hw->h_ifname);
                 aa_mapping_unregister_mapping(lldp, hw, m);
             }
             free(m);
@@ -726,8 +712,7 @@ lldp_put_packet(struct lldp *lldp, struct dp_packet *packet,
 {
     struct lldpd *mylldpd = lldp->lldpd;
     struct lldpd_hardware *hw = lldpd_first_hardware(mylldpd);
-    static const struct eth_addr eth_addr_lldp =
-        { { { 0x01, 0x80, 0xC2, 0x00, 0x00, 0x0e } } };
+    static const struct eth_addr eth_addr_lldp = ETH_ADDR_C(01,80,c2,00,00,0e);
 
     ovs_mutex_lock(&mutex);
 
@@ -739,6 +724,14 @@ lldp_put_packet(struct lldp *lldp, struct dp_packet *packet,
     ovs_mutex_unlock(&mutex);
 }
 
+/* Is LLDP enabled?
+ */
+bool
+lldp_is_enabled(struct lldp *lldp)
+{
+    return lldp ? lldp->enabled : false;
+}
+
 /* Configures the LLDP stack.
  */
 bool
@@ -802,13 +795,7 @@ lldp_create(const struct netdev *netdev,
     ovs_list_init(&lldp->lldpd->g_chassis);
     ovs_list_push_back(&lldp->lldpd->g_chassis, &lchassis->list);
 
-    if ((hw = lldpd_alloc_hardware(lldp->lldpd,
-                                   (char *) netdev_get_name(netdev),
-                                   0)) == NULL) {
-        VLOG_WARN("Unable to allocate space for %s",
-                  (char *) netdev_get_name(netdev));
-        out_of_memory();
-    }
+    hw = lldpd_alloc_hardware(lldp->lldpd, netdev_get_name(netdev), 0);
 
     ovs_refcount_init(&lldp->ref_cnt);
 #ifndef _WIN32
@@ -960,8 +947,8 @@ lldp_ref(const struct lldp *lldp_)
 void
 lldp_destroy_dummy(struct lldp *lldp)
 {
-    struct lldpd_hardware *hw, *hw_next;
-    struct lldpd_chassis *chassis, *chassis_next;
+    struct lldpd_hardware *hw;
+    struct lldpd_chassis *chassis;
     struct lldpd *cfg;
 
     if (!lldp) {
@@ -970,13 +957,13 @@ lldp_destroy_dummy(struct lldp *lldp)
 
     cfg = lldp->lldpd;
 
-    LIST_FOR_EACH_SAFE (hw, hw_next, h_entries, &cfg->g_hardware) {
+    LIST_FOR_EACH_SAFE (hw, h_entries, &cfg->g_hardware) {
         ovs_list_remove(&hw->h_entries);
         free(hw->h_lport.p_lastframe);
         free(hw);
     }
 
-    LIST_FOR_EACH_SAFE (chassis, chassis_next, list, &cfg->g_chassis) {
+    LIST_FOR_EACH_SAFE (chassis, list, &cfg->g_chassis) {
         ovs_list_remove(&chassis->list);
         free(chassis);
     }
