@@ -93,6 +93,7 @@ void ovsdb_weak_ref_destroy(struct ovsdb_weak_ref *);
 
 struct ovsdb_row *ovsdb_row_create(const struct ovsdb_table *);
 struct ovsdb_row *ovsdb_row_clone(const struct ovsdb_row *);
+struct ovsdb_row *ovsdb_row_datum_clone(const struct ovsdb_row *);
 void ovsdb_row_destroy(struct ovsdb_row *);
 
 uint32_t ovsdb_row_hash_columns(const struct ovsdb_row *,
@@ -118,6 +119,7 @@ struct ovsdb_error *ovsdb_row_from_json(struct ovsdb_row *,
     OVS_WARN_UNUSED_RESULT;
 struct json *ovsdb_row_to_json(const struct ovsdb_row *,
                                const struct ovsdb_column_set *include);
+void ovsdb_row_to_string(const struct ovsdb_row *, struct ds *);
 
 static inline const struct uuid *
 ovsdb_row_get_uuid(const struct ovsdb_row *row)
@@ -128,6 +130,7 @@ ovsdb_row_get_uuid(const struct ovsdb_row *row)
 static inline struct uuid *
 ovsdb_row_get_uuid_rw(struct ovsdb_row *row)
 {
+    ovsdb_datum_unshare(&row->fields[OVSDB_COL_UUID], &ovsdb_type_uuid);
     return &row->fields[OVSDB_COL_UUID].keys[0].uuid;
 }
 
@@ -140,6 +143,7 @@ ovsdb_row_get_version(const struct ovsdb_row *row)
 static inline struct uuid *
 ovsdb_row_get_version_rw(struct ovsdb_row *row)
 {
+    ovsdb_datum_unshare(&row->fields[OVSDB_COL_VERSION], &ovsdb_type_uuid);
     return &row->fields[OVSDB_COL_VERSION].keys[0].uuid;
 }
 
